/*
 * A resource offering support for creating processes.
 *
 * Copyright (C) 2023, 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <systypes/fcntl.h>

#include "process_creator_resource.h"
#include "process_creator_context_resource.h"
#include "process_creator_context_object_server.h"

#include <string.h>
#include <stdio.h>



/* Support for creating processes. */

ProcessCreatorContextResource::ProcessCreatorContextResource(ProcessCreatorResource *creator)
: SimplePager(), _creator(creator)
{
}

ProcessCreatorContextResource::~ProcessCreatorContextResource()
{
}

ipc_server_default_config_type ProcessCreatorContextResource::config()
{
    return config_ProcessCreatorContextObject;
}



/* ProcessCreatorContext interface methods. */

long ProcessCreatorContextResource::start(int argc, l4_cap_idx_t reader,
                                          l4_cap_idx_t writer,
                                          l4_cap_idx_t *process)
{
    /* Obtain the arguments by reading from the shared memory. */

    const char *argv[argc];
    offset_t pos = 0;

    if (!argc)
        return -L4_EINVAL;

    for (int i = 0; i < argc; i++)
    {
        argv[i] = get_string(pos);

        if (argv[i] != NULL)
            pos += strlen(argv[i]) + 1;

        printf("argv[%d] = %s\n", i, argv[i]);
    }

    /* Obtain a reference to a new process and send the reference to the client.
       This must be done in advance of actually starting the program since the
       program may complete before the client gets the reference.

       The reference needs to be shared and its reference count increased to
       prevent the closure of the process monitor. */

    long err = _creator->init_process(process);

    if (!err)
        err = complete_ProcessCreatorContext_start(*process);

    /* Attempt to communicate any error from these activities. */

    if (err)
        return err;

    /* Attempt to actually start the process. Errors cannot be communicated as a
       reply, so a notification is sent via the process monitor instead by the
       process creator. */

    _creator->start(argc, argv, reader, writer, *process);

    return IPC_MESSAGE_SENT;
}

/* vim: tabstop=4 expandtab shiftwidth=4
*/
